{-# LANGUAGE DerivingStrategies #-}
{-|
Module      : Gargantext.Core.LinearAlgebra
Description : Linear Algebra utility functions
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

Linear algebra utility functions to be used across all the Gargantext modules requiring it.
-}

module Gargantext.Core.LinearAlgebra (
  -- * Types
    Index(..)

  -- * Functions
  , createIndices
  ) where

import Prelude
import Data.Map.Strict qualified as M
import Data.Set qualified as S
import Data.Set (Set)
import Data.Map.Strict (Map)
import Data.Foldable (foldl')

newtype Index = Index { _Index :: Int }
  deriving newtype (Eq, Show, Ord, Num, Enum)

createIndices :: Ord t => Map (t, t) b -> (Map t Index, Map Index t)
createIndices = set2indices . map2set
  where
    map2set :: Ord t => Map (t, t) a -> Set t
    map2set cs' = foldl' (\s ((t1,t2),_) -> insert [t1,t2] s ) S.empty (M.toList cs')
      where
        insert as s = foldl' (\s' t -> S.insert t s') s as

    set2indices :: Ord t => Set t -> (Map t Index, Map Index t)
    set2indices s = (M.fromList toIndex', M.fromList fromIndex')
      where
        fromIndex' = zip [0..] xs
        toIndex'   = zip xs [0..]
        xs         = S.toList s
